@@ -7,7 +7,7 @@ use strict;
 
 package XML::Compile::Schema::BuiltInFacets;
 use vars '$VERSION';
-$VERSION = '0.52';
+$VERSION = '0.53';
 use base 'Exporter';
 
 use Log::Report 'xml-compile', syntax => 'SHORT';
@@ -154,8 +154,15 @@ sub _enumeration($$$)
 }
 
 sub _totalDigits($$$)
-{   my $nr = $_[2];
-    sub { sprintf "%${nr}f", $_[0] };
+{   my ($path, undef, $nr) = @_;
+    sub { return $_[0] if $nr >= ($_[0] =~ tr/0-9//);
+          my $val = $_[0];
+          return sprintf "%.${nr}f", $val
+              if $val =~ m/^[+-]?0*(\d)[.eE]/ && length($1) < $nr;
+
+          error __x"decimal too long, got {length} digits max {max} at {where}"
+             , length => ($val =~ tr/0-9//), max => $nr, where => $path;
+    };
 }
 
 sub _fractionDigits($$$)
